;;Para compilar> 			nasm boot.asm -o boot.bin -l boot.lst
;;Para copiar al sector de inicio> 	dd if=boot.bin of=\\.\a:

;       +--------+ 1FE0:7E00
;       |BOOT SEC|
;       |RELOCATE|
;       |--------| 1FE0:7C00
;       |LBA PKT |
;       |--------| 1FE0:7BC0
;       |--------| 1FE0:7BA0
;       |BS STACK|
;       |--------|
;       |4KBRDBUF| used to avoid crossing 64KB DMA boundary
;       |--------| 1FE0:63A0
;       |        |
;       |--------| 1FE0:3000
;       | CLUSTER|
;       |  LIST  |
;       |--------| 1FE0:2000
;       |        |
;       |--------| 0000:7E00
;       |BOOT SEC| overwritten by max 128k FAT buffer
;       |ORIGIN  | and later by max 134k loaded kernel
;       |--------| 0000:7C00
;       |        |
;       |--------|
;       |KERNEL  | also used as max 128k FAT buffer
;       |LOADED  | before kernel loading starts
;       |--------| 0060:0000
;       |        |
;       +--------+

segment .text	;hay un unico segmento de codigo y datos

;; ---------------------------------------------------------
;;<	relocalizacion en memoria del codigo del boot sector	>
;; ---------------------------------------------------------

%define BASE            0x7c00
	org     BASE
	
;; ---------------------------------
;;<	saltar al codigo de arranque	>
;; ---------------------------------
Entry:          jmp     short real_start	; salta la BPB al codigo de inicio
                nop							; por precaucion para que la BPB arranque en el tercer byte
											; no iria si se usara jmp far para el salto
;; -------------------------------------
;;<	tabla con los parametros de la BPB	>
;; -------------------------------------
bsOemName 		db	"IIEDOS-1" 		; OEM label
bsBytesPerSec 	dw	512	 			; bytes/sector, 512 bytes por sector
bsSecPerClust 	db	1      			; sectors/allocation unit, un sector por cluster
bsResSectors    dw	1      			; # reserved sectors, un unico sector reservado
bsFATs          db	2      			; # of fats, 2 FAT's en el diskette
bsRootDirEnts   dw	224      		; # of root dir entries, maximo 224 entradas en el root(224 * 32 Bytes = 7168 Bytes = 14 sectores)
bsSectors       dw	2880      		; # sectors total in image, 2880 x 512 Bytes = 1,44 MBytes
bsMedia         db	0F0h      		; media descrip: fd = 2side9sec, etc... _____________________________________________________________
sectPerFat      dw	9      			; # sectors in a fat, 9 sectores para la FAT ________________________________________________________
sectPerTrack    dw	18      		; # sectors/track, 18 sectores por pista ____________________________________________________________
nHeads          dw	2      			; # heads, 2 cabezas/lados/caras... _________________________________________________________________
nHidden         dd	0      			; # hidden sectors, cero sectores escondidos ________________________________________________________
nSectorHuge     dd	0      			; # sectors if > 65536, cero porque no ocurre _______________________________________________________
drive           dw	0      			; drive number, cero (unico) ________________________________________________________________________
extBoot         db	41      		; extended boot signature, 41 para 'floppy'
volid           dd	00000000h		; Volume Serial Number, ceros para no poner verdura _________________________________________________
vollabel        db	"boot FAT hw"	; Volume Label, una etiqueta para este diskette _____________________________________________________
filesys         db	"FAT12   "		; File System ID 
									;				the bytes at offsets 36h through 3Dh should contain the File System ID of: "FAT12   "
									; 				with three trailing space bytes (20h)

;;----------------------------------------------------------------------------------------------------------------------------------------

%define LOADSEG         0x0060	; ubicacion del segmento de carga del codigo a ejecutar
%define FATBUF          0x2000	; offset para el buffer temporario para la cadena de FAT

	times   0x3E-$+$$ db 0		; directiva que reserva hasta 0x3E con ceros??? no entiendo para que esta..._______________________________

;; ------------------------------------------------
;;<	declara parametros de la FAT y buffer temporal >
;; ------------------------------------------------

; usar bp-Entry+loadseg_xxx genera code mas chico que usando solo loadseg_xxx
; aqui bp se inicializa a Entry, entonces bp-Entry = 0
%define loadsegoff_60   bp-Entry+loadseg_off	; loadsegoff_60 = loadseg_off
%define loadseg_60      bp-Entry+loadseg_seg	; loadseg_60 = loadseg_seg

%define LBA_PACKET		bp-0x40					; paquete con los siguientes datos de la LBA
	%define LBA_SIZE       	word [LBA_PACKET]    	; largo del paquete, deberia ser 10h
	%define LBA_SECNUM     	word [LBA_PACKET+2]  	; numero de sectores a leer
	%define LBA_OFF        	LBA_PACKET+4         	; offset buffer para leer y escribir
	%define LBA_SEG        	LBA_PACKET+6			; segmento buffer para leer y escribir
	%define LBA_SECTOR_0   	word [LBA_PACKET+8 ] 	; numero de sector donde arranca la LBA
	%define LBA_SECTOR_16  	word [LBA_PACKET+10]	;____________________________________________________________________________________
	%define LBA_SECTOR_32  	word [LBA_PACKET+12]	;____________________________________________________________________________________
	%define LBA_SECTOR_48  	word [LBA_PACKET+14]	;____________________________________________________________________________________

	
%define READBUF 0x63A0 					; max 4KB buffer (min 2KB stack), == stacktop-0x1800_____________________________________________
%define READADDR_OFF   BP-0x60-0x1804   ; offset del puntero al buffer
%define READADDR_SEG   BP-0x60-0x1802	; segmento del puntero al buffer

%define PARAMS LBA_PACKET+0x10			; una referencia para definir otros parametros
	%define fat_start       PARAMS+0x2         ; primer sector de la FAT
	%define root_dir_start  PARAMS+0x6         ; primer sector del directorio root
	%define data_start      PARAMS+0x0a        ; primer sector de datos

;; -------------------------------------------------------------------------------------
;;< Punto de entrada "ENTRY", a aqui se inicializa bp que se usa como referencia arriba >
;; -------------------------------------------------------------------------------------

real_start:		; aqui arranca el codigo que se ejecuta

;; ---------------------------------------------------------------------------------
;;<	se mueve el sector de inicio en memoria, de BOOT SEC ORIGIN	a BOOT SEC RELOCATE >
;; ---------------------------------------------------------------------------------
	cli		; deshabilita las interrupciones enmascarables
	cld		; inicializa la bandera de direccion para que se incrementen los indices
			; "Un error comn en la programacin 80x86 es el olvido de iniciar explcitamente la bandera de
			; direccin en el estado correcto." - from: ApuntesIntroArqPC_v2.pdf :)

	; apunta con ds:bp a BOOT SEC ORIGIN = 0000:7C00
	xor     ax, ax		; ax = 0
	mov     ds, ax		; ds = 0
	mov     bp, BASE	; bp = BASE = 0x7c00, 

	; apunta con es:di a BOOT SEC RELOCATE = 1FE0:7C00
	mov     ax, 0x1FE0 	; ax = 0x1FE0
        mov     es, ax		; es = 0x1FE0
        mov     si, bp		; si = bp = BASE = 0x7c00
        mov     di, bp		; di = si = bp = BASE = 0x7c00
			
	; carga en cx la cantidad de palabras a copiar, 512 Bytes = 256 Words
	mov     cx, 0x0100	; cx = 256
			
	; copia cx palabras desde [ds:bp] a [es:di] en un bucle
	rep     movsw	; repite cx veces movsw
		

	; sigue ejecutando desde el codigo recien copiado
	jmp     word 0x1FE0:cont	; -->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-->-
																											;	|
	; loadseg_seg:loadseg_off es un puntero a la posicion donde se va a cargar el siguiente programa, 0000:0060		|
	loadseg_off     dw      0		; loadseg_off = 0000														|
	loadseg_seg     dw      LOADSEG		; loadseg_seg = 0060														|
																											;	|
																											;	|
	; sigue el hilo en el codigo copiado mas arriba en memoria	<--<--<--<--<--<--<--<--<--<--<--<--<--<--<--<--<--<-


	;; -----------------------------------
	;;<	se inicializa los datos de la LBA >
	;; -----------------------------------

cont:
	
	; inicializa es segmento ds y el stack
	mov     ds, ax			; ds = ax = 1FE0
	mov     ss, ax			; ss = ax = 1FE0
	lea     sp, [bp-0x60]		; sp = bp-0x60 = ENTRY - 0x60

	sti		; habilita las interrupciones enmascarables
	
	; inicializa los parametros de la LBA
	mov     [drive], dl     		; BIOS pasa el drive number en DL
	mov     LBA_SIZE, 10h			; largo de la LBA = 10h
	mov     LBA_SECNUM,1    		; numero de sectores a leer
	mov     word [LBA_SEG],ds		; segmento del buffer para la LBA
	mov     word [LBA_OFF],READBUF		; offset   del buffer para la LBA

;  -------------------------------------
; < se calculan algunos datos del disco >
;  -------------------------------------

	mov     si, word [nHidden]		
	mov     di, word [nHidden+2]
	; di:si = numero de sectores escondidos

	add     si, word [bsResSectors]
	adc     di, byte 0              
	; di:si = numero de sectores escondidos + numero de sectores reservados

	; entonces di:si = primer sector de la FAT
	mov     word [fat_start], si
	mov     word [fat_start+2], di
	;-----------------------------------------

	mov     al, [bsFATs]
	; al = numero de FAT's en el disco
        
	cbw		; convierte el byte al en la palabra ax
	; ax = numero de FAT's en el disco
        
	mul     word [sectPerFat]
	; ax = (numero de FAT's en el disco) * (numero de sectores por FAT)
				
	; di:si = numero de sectores escondidos + numero de sectores reservados
	add     si, ax
	adc     di, dx                  
	; di:si = numero de sectores escondidos + numero de sectores reservados + numero sectores de FAT
	
	; entonces di:si = primer sector del directorio root
	mov     word [root_dir_start], si
	mov     word [root_dir_start+2], di
	;---------------------------------------------------

	mov     bx, [bsBytesPerSec]		; bx = 512, cantidad de Bytes por sector
	mov     cl, 5
	shr     bx, cl                  	; divide bx por 32 Bytes, que es lo que ocupa una entrada del directorio
	; bx = cantidad maxima de entradas del directorio root que caben en un sector

	mov     ax, [bsRootDirEnts]		; ax = cantidad maxima de entradas en directorio root
	xor     dx, dx				; dx = 0
	div     bx				; divide dx_ax por bx, ax = cantidad de entradas del directorio root / cantidad de entradas por sector
	; ax = cantidad de sectores que ocupan las entradas del directorio root

        push    ax				; respalda ax en el stack
		
	; di:si = numero de sectores escondidos + numero de sectores reservados + numero sectores de FAT
	add     si, ax			
	adc     di, byte 0
	; di:si = numero de sectores escondidos + numero de sectores reservados + numero sectores de FAT + numero de sectores de root
		
	; entonces di:si es el primer sector de los datos
        mov     [data_start], si
        mov     [data_start+2], di
	;------------------------------------------------

;  -------------------------------------------
; < se busca al archivo en el directorio root >
;  -------------------------------------------

; --> traer todas las entradas del directorio root al buffer temporario en memoria

	; dx:ax = principio del directorio root
	mov     ax, word [root_dir_start]
	mov     dx, word [root_dir_start+2]
	pop     di			; carga en di la cantidad de sectores que ocupan las entradas del directorio root
	les     bx, [loadsegoff_60] 	; es:bx = 0060:0000, un buffer que le pasamos a readDisk para copiar el(los) sector(es)
	
	call    readDisk		; carga en [es:bx] la cantidad de sectores especificada en di, desde [dx:ax]
    
	; ya tenemos todas las entradas del root en el buffer
	les     di, [loadsegoff_60] ; es:di = 0060:0000


; --> buscar el archivo por su nombre y obtener el primer cluster	

next_entry:
	mov     cx, 11		; cx es el largo del nombre de archivo, 11 caracteres
	mov     si, filename 	; carga en si el puntero al nombre de archivo
	push    di		; respalda di
	repe    cmpsb		; compara cada letra del filename con cada letra del nombre en la entrada actual
						; mientras sean iguales y cx > 0...
	pop     di		; recupera di
	mov     ax, [es:di+0x1A]	; ax = numero de cluster desde la entrada actual
					
	je      ffDone		; esto ocurre si cx = 0, o sea se verificaron todas las letras del nombre de archivo

	add     di, byte 0x20   ; pasamos a la siguiente entrada
	cmp     byte [es:di], 0 ; compara con cero el primer byte del nombre de esta nueva entrada
	jnz     next_entry      ; si es cero, no hay mas archivos en el directorio

	jmp      boot_error      ; se recorrieron todas las entradas y no aparecio el archivo
		
ffDone:		; listo, ya tenemos el primer cluster del archivo que buscamos
	push    ax              ; respalda en el stack el primer cluster

;  ---------------------------------------------------------
; < construir la cadena de clusters que componen el archivo >
;  ---------------------------------------------------------

; Lee la cadena FAT y la guarda en un buffer temporal en los primeros 64 kb.
; La cadena se guarda en un arreglo de 16-bits que contiene los numeros de cluster.

; El archivo tiene que entrar en la memoria convencional de 640 KB y cada sector es de 512 Bytes
; Entonces la cadena no puede ser mayor que (640 KBytes / 512Bytes) * 2 = 2560 = 2.5 KByte

	les     bx, [loadsegoff_60]     ; es:bx = 0060:0000
	mov     di, [sectPerFat]	; di = cantidad de sectores que ocupa la FAT
	mov     ax, word [fat_start]	; carga desde los parametros de la LBA
	mov     dx, word [fat_start+2]	; dx_ax = primer sector de la FAT
	call    readDisk		; carga en [es:bx] la cantidad de sectores especificada en di, desde [dx:ax]

	pop     ax			; recupera el primer cluster del archivo desde el stack

	; es:di apunta al buffer temporario de la FAT
	push    ds			; respalda el segmento ds = 1FE0 en el stack
	pop     es 			; es = ds = 1FE0
	mov     ds, [loadseg_60]	; carga en ds el segmento del buffer de la FAT
	mov     di, FATBUF		; carga el offset para el buffer temporario de la fat

next_clust:     
	stosw				; guarda en el buffer el numero de cluster en ax
	mov     si, ax  		; si = ax = numero de cluster

fat_12:     
	add     si, si          ; multiplica el numero de cluster por 3
	add     si, ax		; se lo suma al mismo numero de cluster
	shr     si, 1           ; y se divide por 2
	lodsw			; carga el siguiente desde el buffer en ax

				; si el numero de cluster era impar, el valor del cluster esta en los bits 0-11 de ax
				; si el numero de cluster era par,   el valor del cluster esta en los bits 4-15 de ax
				; entonces si era par se enmascara y listo
				; si era impar hay que rotarlo (poniendo ceros) y listo.
				; la bandera cf se setea al hacer la division anterior
				
	jnc     fat_even	; salto si es un cluster impar
	mov     cl, 4		; cl es la cantidad de bits a rotar
	shr     ax, cl		; rota ax hacia a la derecha, cl veces, poniendo ceros

fat_even:   
	and     ah, 0x0f        ; enmascara ah con 00001111 = 0f
	cmp     ax, 0x0ff8      ; compara con 0FF8 = 16-bit EOF
	jb      next_clust      ; continua si no es el ultimo cluster

	
	; aqui se termino el archivo
	xor     ax, ax
	stosw			; luego guardamos un ultimo numero de cluster = 0

	push    cs		; respalda el segmento de codigo en el stack
	pop     ds		; recupera el segmento de datos desde el stack
	; queda cs = ds = 1FE0

;  ----------------------------------------------------------------------
; < carga el archivo a memoria, cluster por cluster y le pasa el control >
;  ----------------------------------------------------------------------

	les     bx, [loadsegoff_60]   	; pone es:bx apuntando a la direccion de carga 0060:0000
	mov     si, FATBUF      	; pone ds:si apuntando a la cadena de FAT

cluster_next:   
	lodsw				; carga desde el buffer FAT, ax = proximo cluster a leer
	or      ax, ax			; se fija si es el fin de archivo
	jne     load_next		; si no es, continua cargando el sector correspondiente
                
					; se ajusta el salto segun el largo del encabezado del .exe
	les	bx,[loadsegoff_60]	; es:bx es la primer linea del .exe en memoria
	add	bx,8			; ahora es:bx es el puntero al parametro
	mov ax,[es:bx]			; carga en ax el parametro a sumar
	add[loadseg_60],ax		; suma el parametro al segmento de inicio

	jmp     far [loadsegoff_60] 	; fue!, ahora manda el archivo cargado

load_next:  
	dec     ax			; cluster numbers start with 2_______________________________________________________________________
	dec     ax			; cluster numbers start with 2_______________________________________________________________________				

	mov     di, word [bsSecPerClust]	; cantidad de sectores a cargar por cluster
	and     di, 0xff                	; enmascara porque solo importa el lsByte, di = sectors per cluster
	mul     di				; multiplica di por ax
	add     ax, [data_start]		; suma eax con el primer sector de datos
	adc     dx, [data_start+2]      	; dx:ax = sector a leer
					
	call    readDisk	; carga en [es:bx] la cantidad de sectores especificada en di, desde [dx:ax]
				
	jmp     short cluster_next		; vuelve a fijarse si tiene que cargar otro cluster

;________________________________________________________________________________
; fin del codigo de inicio, de aqui en mas estan las funciones usadas hasta ahora
;________________________________________________________________________________
					
				
;  -------------------------------------------------------------------------
; < imprime un texto terminado en '.' ubicado luego del call a esta funcion >
;  -------------------------------------------------------------------------
show:   pop     si			; respalda la direccion de retorno en si
        lodsb                           ; toma el caracter
        push    si                      ; devuelve la direccion de retorno al stack
        mov     ah,0x0E                 ; funcion BIOS 0eH, "mostrar caracter"
        int     0x10                    ; de la interrupcion 10h, via "TTY" mode
        cmp     al,'.'                  ; compara con el punto esperado al fin del mensaje
        jne     show                    ; continua leyendo y mostrando caracter por caracter hasta que venga el esperado punto
        ret								; vuelve a donde llamaron

		
;  ---------------------------------------------------------------
; < imprime un mensaje de error si no se puede iniciar el sistema >
;  ---------------------------------------------------------------
boot_error:     call    show
                db      "chanfle!."

                xor     ah,ah		; funcion 00h
                int     0x13       	; resetear la disketera
                int     0x16        ; esperar una tecla
                int     0x19        ; reiniciar la maquina

;  ----------------------------------------------------------------------------
; < carga en [es:bx] la cantidad de sectores especificada en di, desde [dx:ax] >
;  ----------------------------------------------------------------------------
readDisk:       push    si							; respalda si en el stack

                mov     LBA_SECTOR_0,ax				
                mov     LBA_SECTOR_16,dx
                mov     word [READADDR_SEG], es
                mov     word [READADDR_OFF], bx

                call    show
                db      "."
	read_next:

                mov     cx,LBA_SECTOR_0		; cx = LBA_SECTOR_0
                mov     dx,LBA_SECTOR_16	; dx = LBA_SECTOR_16
                
                ; --> tranformar el numero de sector en los parametros de BIOS (C,H,S)
                ;
                ; abs = sector                          offset en pista
                ;     + head * sectPerTrack             offset en cilindro
                ;     + track * sectPerTrack * nHeads   offset en plato
                ;
                mov     al, [sectPerTrack]				; al = sectores por pista
                mul     byte [nHeads]					; ax = al * cabezas
                xchg    ax, cx							; ax <--> cx,
                ; cx = nHeads * sectPerTrack <= 255*63
                ; dx:ax = abs
                div     cx								; divide dx_ax por cx
                ; ax = track, dx = sector + head * sectPertrack
                xchg    ax, dx							; ax <--> dx
                ; dx = track, ax = sector + head * sectPertrack
                div     byte [sectPerTrack]				;divide ax por el Byte en [sectPerTrack]
                ; dx =  track, al = head, ah = sector
                mov     cx, dx							; cx = dx
                ; cx =  track, al = head, ah = sector

				; las siguientes manipulaciones son necesarias para colocar en orden los datos en los registros
                ; ch = numero de cilindro, 8 bits menos significativos
                ; cl = 7-6: cilindro, 2 bits mas significativos
                ;      5-0: sector
					mov     dh, al                  ; guarda cabezal en dh
					xchg    ch, cl                  ; ch <--> cl
					ror     cl, 1                   ; mueve los bits mas significativos de la pista 
					ror     cl, 1                   ; a los bits 7-6
					or      cl, ah                  ; junta sector con cilindro
					inc     cx                      ; pasa al modo de sectores "1-based (1-63)"

					les     bx,[LBA_OFF]	; apunta con es:bx al buffer 
					mov     ax, 0x0201		; carga el servicio 02h (en ah) para leer 01h  (en al) sectores
	
		do_int13_read:                
						mov     dl, [drive]				; carga en dl el numero de dispositivo
						int     0x13					; solicita la interrupcion 13h
						jc      boot_error              ; no se pudo leer, salida con mensaje de error
				
						mov     ax, word [bsBytesPerSec]  ; ax = numero de Bytes por sector (512 Bytes) 

						push    di						; respalda di en el stack
						mov     si,READBUF              ; apunta con ds:si al buffer de origen de los datos
						les     di,[READADDR_OFF]       ; apunta con es:di al buffer de destino
						mov     cx, ax					; pone la cantidad de Bytes a copiarse en cx

						rep     movsb					; copia en [ES:DI] cx Bytes desde [DS:SI]
						pop     di						; recupera di desde el stack

						mov     cl, 4					; cl = 4
						shr     ax, cl                  ; ajusta el segmento puntero incrementando
						add     word [READADDR_SEG], ax ; por paragrafo leido (por sector)

						add     LBA_SECTOR_0,  byte 1	
						adc     LBA_SECTOR_16, byte 0   ; dx:ax = proximo sector a leer
						dec     di                      ; si queda algo por leer...
						jnz     read_next               ; continua

						les     bx, [READADDR_OFF]		; ya esta, asi que hace es:bx = [READADDR_OFF]
						pop     si						; recupera si desde el stack
ret

times   0x01f1-$+$$ db 0		; completa con ceros hasta los 497 Bytes de un sector de 512, dejando lugar para:
filename db "KERNEL  EXE",0,0	; nombre del archivo que se busca, carga y pasa el control

sign            dw      0xAA55	; numero magico, firma de un sector de inicio
