segment KNT 		class=CODE align=16 USE16
segment KND 		class=FAR_DATA align=16
segment KND_DATA        class=FAR_DATA align=2
segment KNDCONST        class=FAR_DATA align=2
segment KNDCONST2       class=FAR_DATA align=2
segment KND_BSS         class=BSS align=2
segment KND_STACK	class=STACK align=2

group KND_GROUP KND KNDCONST KNDCONST2 KND_DATA KND_BSS KND_STACK

%define STACK_SIZE 1000h

;CPU 8086

extern 	_end
extern	_edata

segment KNT

global _cstart_
..start:
_cstart_:
	mov	ax,cs
	add 	ax,KND_GROUP
	mov	ds,ax
	mov	es,ax
	
	cli	
	mov	ss,ax
	mov	sp,_end + STACK_SIZE
	sti

	cld
	xor	ax,ax
	mov	di, _edata	; Start of bss is at end of data
	mov	cx, _end	; End of bss (begin of heap)
	sub	cx, di		; Number of bss bytes
	shr	cx, 1		; Number of words
	rep	stosw		; Clear bss

	mov	ax,cs
	mov	[_code_seg],ax
	mov	ax,ds
	mov	[_data_seg],ax
	mov	ax,ss
	mov	[_stack_seg],ax
	
	mov [_knd_group], word KND_GROUP
	mov	[_linker_end], word _end
	mov	[_linker_edata], word _edata
	mov	[_spini], sp

	extern 	main_
	call 	main_

infinito:
	jmp infinito


;;;
;Aqui voy a agregar la funcion putk(int c)
;el entero c es el ascii correspondiente al caracter a imprimir
;lo tengo que levantar desde el stack, cuidando saltar la direccion de retorno

global _putk
_putk:
        push    bp
        mov     bp,sp
        pusha
        mov     al, [bp+4]      ; al = caracter impreso
        test    al, al          ; caracter nulo terminando el string?

        jz      nulch
        cmp     al, 0Ah         ; al = nueva linea? El compilador inserta 0Ah en lugar de '\n' en un string
        jnz     putc
        mov     al, 0Dh         ; Si al == nueva linea == 0Ah, imprimo 0Dh (Caracter Return) previo al fin de linea
        mov     ah, 0Eh         ; Imprime caracter en modo texto
        mov     bx, 0007        ; Pagina 0, blanco sobre negro
        int     10h             ; BIOS VIDEO_IO
        mov     al, 0Ah         ; Restauro el caracter de fin de linea y lo imprimo
putc:
        mov     ah, 0Eh         ; Imprime caracter en modo texto
        mov     bx, 0007        ; Pagina 0, blanco sobre negro
        int     10h             ; BIOS VIDEO_IO
nulch:
        popa
        pop     bp
        ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;

global _getchar
; int getchar(void), peekchar(void);
;	Read a character from the keyboard, or just look if there is one.
;	A carriage return is changed into a linefeed for UNIX compatibility.
_getchar:
	xor	ah, ah		; Read character from keyboard
	int	16h
getc:	cmp byte al, 0Dh	; Carriage return?
	jnz	nocr
	mov	al, 0Ah		; Change to linefeed
nocr:	xor	ah, ah		; ax = al
	ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;uint32_t DFS_ReadSector_FD(uint8_t unit, uint8_t *buffer, uint16_t cylinder, uint16_t head, uint16_t sector)
global _DFS_ReadSector_FD
extern	_reg_dump
_DFS_ReadSector_FD:
	push	si 
	push	di 
	push	bp 
	mov	bp,sp 

	mov	dl,[bp+08h]

	; cx =  track, al = head, ah = sector
	mov	cx,[bp+0ch]	;cylinder
	mov	ax,[bp+0eh]	;head
	mov	dh,al		;dh = head
	mov	ax,[bp+10h]	;sector
	mov	ah,al		;ah = sector
	mov	al,dh		;al = head


; ch = numero de cilindro, 8 bits menos significativos
; cl = 7-6: cilindro, 2 bits mas significativos
;      5-0: sector
	mov     dh, al                  ; guarda cabezal en dh
	xchg    ch, cl                  ; ch <--> cl
	ror     cl, 1                   ; mueve los bits mas significativos de la pista 
	ror     cl, 1                   ; a los bits 7-6
	or      cl, ah                  ; junta sector con cilindro

	mov     bx,[bp+0ah]		; apunta con es:bx al buffer 
	mov     ax, 0x0201		; carga el servicio 02h (en ah) para leer 01h  (en al) sectores

;	push	dx
;	push	cx
;	push	bx
;	push	ax
;	call	_reg_dump
;	pop	ax
;	pop	bx
;	pop	cx
;	pop	dx

	int     13h			; solicita la interrupcion 13h
	jc      fdread_error              ; no se pudo leer, salida con mensaje de error

	mov	ax,0001 
	mov	dx,0000 
	jmp	fdread_end

fdread_error:

	mov	ax,0000 
	mov	dx,0000 

fdread_end:
	mov         sp,bp 
	pop         bp 
	pop         di 
	pop         si 
	ret



; Esta es la funcion que copia el codigo desde el buffer al destino en memoria donde luego es ejecutado.
;*				phys_copy				     *
;*===========================================*
; This routine copies a block of physical memory.  It is called by:
;    phys_copy( (long) source, (long) destination, (long) bytecount)

global _phys_copy
_phys_copy:
	pushf			; save flags
	cli				; disable interrupts
	push bp			; save the registers
	push ax			; save ax
	push bx			; save bx
	push cx			; save cx
	push dx			; save dx
	push si			; save si
	push di			; save di
	push ds			; save ds
	push es			; save es
	mov bp,sp		; set bp to point to saved es

  L0:	mov ax,[bp + 28]	; ax = high-order word of 32-bit destination
	mov di,[bp + 26]		; di = low-order word of 32-bit destination
	mov cx,4			; start extracting click number from dest
  L1:	rcr ax,1		; click number is destination address / 16
	rcr di,1			; it is used in segment register for copy
	loop L1				; 4 bits of high-order word are used
	mov es,di			; es = destination click

	mov ax,[bp + 24]		; ax = high-order word of 32-bit source
	mov si,[bp + 22]		; si = low-order word of 32-bit source
	mov cx,4			; start extracting click number from source
  L2:	rcr ax,1		; click number is source address / 16
	rcr si,1			; it is used in segment register for copy
	loop L2				; 4 bits of high-order word are used
	mov ds,si			; ds = source click

	mov di,[bp + 26]		; di = low-order word of dest address
	and di,0x000F		; di = offset from paragraph # in es
	mov si,[bp + 22]		; si = low-order word of source address
	and si,0x000F		; si = offset from paragraph # in ds

	mov dx,[bp + 32]		; dx = high-order word of byte count
	mov cx,[bp + 30]		; cx = low-order word of byte count

	test cx,0x8000		; if bytes >= 32768, only do 32768 
	jnz L3				; per iteration
	test dx,0xFFFF		; check high-order 17 bits to see if bytes
	jnz L3				; if bytes >= 32768 then go to L3
	jmp L4				; if bytes < 32768 then go to L4
  L3:	mov cx,0x8000	; 0x8000 is unsigned 32768
  L4:	mov ax,cx		; save actual count used in ax; needed later

	test cx,0x0001		; should we copy a byte or a word at a time?
	jz L5				; jump if even
	rep					; copy 1 byte at a time
	movsb				; byte copy
	jmp L6				; check for more bytes

  L5:	shr cx,1		; word copy
	rep					; copy 1 word at a time
	movsw				; word copy

  L6:	mov dx,[bp + 32]	; decr count, incr src & dst, iterate if needed
	mov cx,[bp + 30]		; dx || cx is 32-bit byte count
	xor bx,bx			; bx || ax is 32-bit actual count used
	sub cx,ax			; compute bytes - actual count
	sbb dx,bx			; dx || cx is # bytes not yet processed
	or cx,cx			; see if it is 0
	jnz L7				; if more bytes then go to L7
	or dx,dx			; keep testing
	jnz L7				; if loop done, fall through

	
	pop es			; restore all the saved registers
	pop ds			; restore ds
	pop di			; restore di
	pop si			; restore si
	pop dx			; restore dx
	pop cx			; restore cx
	pop bx			; restore bx
	pop ax			; restore ax
	pop bp			; restore bp
	popf			; restore flags
	
	ret				; return to caller

L7:	mov [bp + 32],dx		; store decremented byte count back in mem
	mov [bp + 30],cx		; as a long
	add [bp + 26],ax		; increment destination
	adc [bp + 28],bx		; carry from low-order word
	add [bp + 22],ax		; increment source
	adc [bp + 24],bx		; carry from low-order word
	jmp L0					; start next iteration

	

; The following constants are offsets into the proc table.
esreg_idx equ 14
dsreg_idx equ 16
csreg_idx equ 18
ssreg_idx equ 20
SP_idx    equ 22
PC_idx    equ 24
PSW_idx   equ 28
SPLIM_idx equ 50
OFF_idx   equ 18
ROFF_idx  equ 12
;K_STACK_BYTES equ  256
K_STACK_BYTES equ  2048

extern _k_stack
;*===========================================================================*
;*				save					     *
;*===========================================================================*
save:				; save the machine state in the proc table.  
	push ds			; stack: psw/cs/pc/ret addr/ds
	push ax			; prepare to restore ds
	mov  ax,cs		
	add ax,KND_GROUP
	mov ds,ax		; ds has now been set to cs + KND_GROUP
	pop ax			
	pop word [ds_save]	; stack: psw/cs/pc/ret addr
	pop word [ret_save]	; stack: psw/cs/pc
	mov [bx_save],bx	; save bx for later ; we need a free register
	mov bx,[_proc_ptr]	; start save set up; make bx point to save area
	add bx,OFF_idx		; bx points to place to store cs
	pop word [bx-OFF_idx+PC_idx]		; store pc in proc table 
	pop word [bx-OFF_idx+csreg_idx]	; store cs in proc table
	pop word [bx-OFF_idx+PSW_idx]	; store psw 
	mov [bx-OFF_idx+ssreg_idx],ss	; store ss
	mov [bx-OFF_idx+SP_idx],sp	; sp as it was prior to interrupt
	mov sp,bx		; now use sp to point into proc table/task save
	mov bx,ds		; about to set ss
	mov ss,bx		; set ss
	push word [ds_save]	; start saving all the registers, sp first
	push es			; save es between sp and bp
	mov es,bx		; es now references kernel memory too
	push bp			; save bp
	push di			; save di
	push si			; save si
	push dx			; save dx
	push cx			; save cx
	push word [bx_save]	; save original bx
	push ax			; all registers now saved
	mov sp,_k_stack		; temporary stack for interrupts
	add sp,K_STACK_BYTES	; set sp to top of temporary stack
	mov ax,[ret_save]	; ax = address to return to
	push ax
	ret
	;jmp (ax)		; return to caller; Note: sp points to saved ax


global _restart
_restart:			; This routine sets up and runs a proc or task.
	cli			; disable interrupts
	mov sp,[_proc_ptr]	; return to user, fetch regs from proc table
	pop ax			; start restoring registers
	pop bx			; restore bx
	pop cx			; restore cx
	pop dx			; restore dx
	pop si			; restore si
	pop di			; restore di
	mov [lds_low],bx	; lds_low contains bx
	mov bx,sp		; bx points to saved bp register
	mov bp,[bx+dsreg_idx-ROFF_idx]	; bp = ds
	mov [lds_low+2],bp	; lds_low+2 contains ds
	pop bp			; restore bp
	pop es			; restore es
	mov sp,[bx+SP_idx-ROFF_idx]	; restore sp
	mov ss,[bx+ssreg_idx-ROFF_idx]	; restore ss using the value of ds
	push word [bx+PSW_idx-ROFF_idx]	; push psw
	push word [bx+csreg_idx-ROFF_idx]	; push cs
	push word [bx+PC_idx-ROFF_idx]	; push pc
	lds bx,[lds_low]		; restore ds and bx in one fell swoop
	iret			; return to user or task




extern _syscall
global _s_call
;*===========================================================================*
;*				s_call					     *
;*===========================================================================*
_s_call:			; System calls are vectored here.
	call 	save
	call	_syscall
	jmp	_restart
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


global _get_byte
_get_byte:
	push	bp
	mov	bp,sp
	push	ds
	push	si
	
	mov	ax,[bp+4]
	mov	si,[bp+6]
	mov	ds,ax
	lodsb
	xor byte ah,ah
	
	pop	si
	pop	ds
	pop	bp
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
global	_exec_command_com
global	_exec_command_com_ret
_exec_command_com:
	mov	dx,CommandComPath
	mov	ah,04bh
	int	21h
_exec_command_com_ret:
	ret


Segment KNDCONST
CommandComPath		db  'COMMAND.COM',00h

Segment KND_BSS
bx_save		dw 0		; storage for bx
ds_save		dw 0		; storage for ds
ret_save	dw 0		; storage for return address
lds_low		dw 0,0		; storage used for restoring bx

extern 	_linker_end
extern	_linker_edata
extern	_spini
extern	_knd_group        
extern	_code_seg
extern	_data_seg
extern	_stack_seg
extern	_proc
extern	_proc_ptr