#include "const.h"
#include "syscall.h"
#include "globales.h"
#include "fs.h"
#include "misc.h"
#include "proc.h"
#include "asminc.h"
#include "printk.h"


uint16_t syscall()
{
	int ax;
	ax = proc_ptr->p_reg[REG_AX];
	ax = ax & 0xff00;
	ax = ax >> 8;
	/*printk("\nInterrupcion 21h ejecutada con parametro %Xh\n", ax);*/

	switch (ax)
	{
		case TEST_CODE:
			printk("first_click = %Xh\n", first_click);
			break;
		case KEYBOARDIN_WITHECHO_CODE:
			kb_in_wecho();
			break;
		case DISPLAY_OUTPUT_CODE:
			display_output();
			break;
		case DISPLAY_STRING_CODE:
			display_string();
			break;
		case TERMINATE_CODE:
			printk("Terminando proceso\n");
			terminate();
			break;
		case EXEC_CODE:
			exec_exe();
			break;
		default:
			printk("Funcion no implementada\n");
			break;
	};

	return 1;
};

uint16_t terminate()
{
	uint16_t wError;
	wError = 0;
	if (proc_ptr->p_parent != 0)
	{
		proc_ptr = proc_ptr->p_parent;
	}
	else
	{
		panic("Error terminate\n");
	};
	return wError;
};

uint16_t exec_com()
{
	uint16_t wError;
	uint8_t buffer[SECTOR_SIZE];
	uint8_t unit;
	uint32_t count;
	unsigned int *p;
	VOLINFO vi;
	uint8_t sector[SECTOR_SIZE];
	FILEINFO fi;
	long phys_dst;
	long phys_src;
	unsigned linear_src_offset, linear_src_segment;
	unsigned linear_dst_offset, linear_dst_segment;
	int nProc;
	char c;
	char szpath[MAX_PATH];
	int ax, ds, dx, i;

	wError = 0;

	ax = proc_ptr->p_reg[REG_AX];
	ds = proc_ptr->p_reg[REG_DS];
	dx = proc_ptr->p_reg[REG_DX];
	ax = ax & 0xff00;
	ax = ax >> 8;

	i = 0;
	for (i=0; i<MAX_PATH; i++)
	{
		szpath[i] = get_byte(ds, dx+i);
		if (szpath[i] == 0)
			break;
	};
	if (i >= MAX_PATH)
		panic("Error EXEC path\n");

	printk("Path to EXEC: %s\n", szpath);

	if (first_click + COM_MEM_SIZE_CLICKS > MAX_MEM_CLICK)
		return EXEC_COM_ERR_INSUFICIENTMEM;

	for (i=0; i<SECTOR_SIZE; i++)
		buffer[i] = 0;

	p = (unsigned int*)(buffer + 510);

	printk("Signature ini: %X\n", *p);

	if (DFS_ReadSector(0, buffer, 0, 1))
	{
		printk("Error leyendo sector de disco");
	}
	else
	{
		/*printk("Signature end: %X\n", *p);*/
	};

	if (DFS_GetVolInfo(0, buffer, 0, &vi))
	{
		printk("Error getting volume information\n");
	}
	else
	{
/*		printk("Volume label '%-11.11s'\n", vi.label);

		printk("%d sector/s per cluster, ", vi.secperclus);
		printk("%d reserved sector/s, ", vi.reservedsecs);
		printk("volume total %d sectors.\n", vi.numsecs);

		printk("%d sectors per FAT, ", vi.secperfat);
		printk("first FAT at sector #%d, ", vi.fat1);
		printk("root dir at #%d.\n", vi.rootdir);

		printk("(For FAT32, the root dir is a CLUSTER number, FAT12/16 it is a SECTOR number)\n");
		printk("%d root dir entries, ", vi.rootentries);
		printk("data area commences at sector #%d.\n",vi.dataarea);
		printk("%d clusters ", vi.numclusters);
		printk("(%d bytes) in data area, filesystem IDd as ", vi.numclusters * vi.secperclus * SECTOR_SIZE);
		if (vi.filesystem == FAT12)
			printk("FAT12.\n");
		else
			printk("[unknown]\n");*/
	};

	if (DFS_OpenFile(&vi, szpath, DFS_READ, sector, &fi))
			printk("Error abriendo archivo file\n");
	else
		{
			unsigned long i;
			DFS_ReadFile(&fi, sector, filebuffer, &i, fi.filelen);
		};

	printk("Cargando buffer a first_click\n");

	linear_src_offset = (unsigned int)filebuffer;
	linear_src_segment = data_seg;
	phys_src = logic2phys(linear_src_segment, linear_src_offset);
	printk("\phys_src = %lxh\n", phys_src);
	printk("first_click = %Xh\n", first_click);

	linear_dst_offset = 0x100;
	linear_dst_segment = first_click;
	phys_dst = logic2phys(linear_dst_segment, linear_dst_offset);
	phys_copy(phys_src, phys_dst, fi.filelen);

	nProc = FIRST_USER_PROC;
	for (i=0; i<NR_PROCS; i++)
	{
		if (proc[i].p_flags & P_SLOT_FREE)
		{
			nProc = i;
			break;
		};
	};

	printk("Primer proceso libre: %X\n", nProc);

	for (i=0; i<REG_ES; i++)
		proc[nProc].p_reg[i] = 0x0000;

	proc[nProc].p_reg[REG_ES] = first_click;
	proc[nProc].p_reg[REG_DS] = first_click;
	proc[nProc].p_reg[REG_CS] = first_click;
	proc[nProc].p_reg[REG_SS] = first_click;

	proc[nProc].p_pcpsw.pc = (int (*)())0x0100;
	proc[nProc].p_pcpsw.psw = INIT_PSW;
	proc[nProc].p_sp = 0;
	proc[nProc].p_flags = 0;

	proc[nProc].p_parent = proc_ptr;

	first_click += COM_MEM_SIZE_CLICKS;
	proc_ptr = &proc[nProc];

	printk("first_click a la salida= %Xh\n", first_click);

	return wError;
}


uint16_t exec_exe()
{
	uint16_t wError;
	uint8_t buffer[SECTOR_SIZE];
	uint8_t unit;
	uint32_t count;
	unsigned int *p;
	int *pi;
	VOLINFO vi;
	uint8_t sector[SECTOR_SIZE];
	FILEINFO fi;
	long phys_dst;
	long phys_src;
	unsigned linear_src_offset, linear_src_segment;
	unsigned linear_dst_offset, linear_dst_segment;
	int nProc;
	char c;
	char szpath[MAX_PATH];
	int ax, ds, dx, i;
	int cs, sp, ss;
	int	psp_size_clicks;
	unsigned int un_head_size;

	psp_size_clicks = 0x10;

	wError = 0;

	ax = proc_ptr->p_reg[REG_AX];
	ds = proc_ptr->p_reg[REG_DS];
	dx = proc_ptr->p_reg[REG_DX];
	ax = ax & 0xff00;
	ax = ax >> 8;

	i = 0;
	for (i=0; i<MAX_PATH; i++)
	{
		szpath[i] = get_byte(ds, dx+i);
		if (szpath[i] == 0)
			break;
	};
	if (i >= MAX_PATH)
		panic("Error EXEC path\n");

	printk("Path to EXEC: %s\n", szpath);

	if (first_click + COM_MEM_SIZE_CLICKS > MAX_MEM_CLICK)
		return EXEC_COM_ERR_INSUFICIENTMEM;

	for (i=0; i<SECTOR_SIZE; i++)
		buffer[i] = 0;

	p = (unsigned int*)(buffer + 510);

	printk("Signature ini: %X\n", *p);

	if (DFS_ReadSector(0, buffer, 0, 1))
	{
		printk("Error leyendo sector de disco");
	}

	if (DFS_GetVolInfo(0, buffer, 0, &vi))
	{
		printk("Error getting volume information\n");
	}

	if (DFS_OpenFile(&vi, szpath, DFS_READ, sector, &fi))
			printk("Error abriendo archivo file\n");
	else
		{
			unsigned long i;
			DFS_ReadFile(&fi, sector, filebuffer, &i, fi.filelen);
		};

	pi = (int*)(filebuffer+8);
	un_head_size = ((unsigned int)(*pi))*16;
/*	printk("Header size: %x\n", un_head_size);	*/
	cs = psp_size_clicks;
	pi = (int*)(filebuffer+0x0E);
	ss = cs + (*pi);
	pi = (int*)(filebuffer+0x10);
	sp = (*pi);

	printk("cs: %x, ",cs);
	printk("ss: %x, ",ss);
	printk("sp: %x\n",sp);

	printk("Cargando programa a first_click\n");
/*	printk("Primer byte del buffer: %x\n", *(filebuffer+un_head_size));*/

	linear_src_offset = (unsigned int)(filebuffer+un_head_size);
	linear_src_segment = data_seg;
	phys_src = logic2phys(linear_src_segment, linear_src_offset);
	printk("\phys_src = %lxh\n", phys_src);
	printk("first_click = %Xh\n", first_click);

	linear_dst_offset = 0x100;			/*PSP*/
	linear_dst_segment = first_click;
	phys_dst = logic2phys(linear_dst_segment, linear_dst_offset);
	phys_copy(phys_src, phys_dst, fi.filelen - (uint32_t)un_head_size);

	nProc = FIRST_USER_PROC;
	for (i=0; i<NR_PROCS; i++)
	{
		if (proc[i].p_flags & P_SLOT_FREE)
		{
			nProc = i;
			break;
		};
	};

	printk("Primer proceso libre: %X\n", nProc);

	for (i=0; i<REG_ES; i++)
		proc[nProc].p_reg[i] = 0x0000;

	proc[nProc].p_reg[REG_ES] = first_click;
	proc[nProc].p_reg[REG_DS] = first_click;
	proc[nProc].p_reg[REG_CS] = first_click + cs;
	proc[nProc].p_reg[REG_SS] = first_click + ss;

	proc[nProc].p_pcpsw.pc = (int (*)())0x0000;
	proc[nProc].p_pcpsw.psw = INIT_PSW;
	proc[nProc].p_sp = (int *)sp;
	proc[nProc].p_flags = 0;

	proc[nProc].p_parent = proc_ptr;

	first_click += COM_MEM_SIZE_CLICKS;
	proc_ptr = &proc[nProc];

/*	proc_dump(&proc[nProc]);*/

	printk("first_click a la salida= %Xh\n", first_click);

	return wError;
}

uint16_t display_string()
{
	uint16_t wError;
	char buffer[SECTOR_SIZE];
	int ax, ds, dx, i;

	wError = 0;

	ds = proc_ptr->p_reg[REG_DS];
	dx = proc_ptr->p_reg[REG_DX];

	i = 0;
	for (i=0; i<SECTOR_SIZE; i++)
	{
		buffer[i] = get_byte(ds, dx+i);
		if (buffer[i] == '$')
		{
			buffer[i] = 0;
			break;
		};
	};

	if (i >= SECTOR_SIZE)
		panic("Error display_string_size\n");

	printk("%s",buffer);

	return wError;
};


uint16_t kb_in_wecho()
{
	int c;
	c= getchar();

	if ((c == '\b') || (c == '\177') || (c == '\25') || (c == '\30'))
	{
		/* Backspace, DEL, ctrl-U, or ctrl-X. */
		printk("\b \b");
	}
	else if (c < ' ' && c != '\n')
	{
		putk('\7');
	}
	else
		putk(c);

	proc_ptr->p_reg[REG_AX] = c;

	return 0;
};

uint16_t display_output()
{
	int dx;
	dx = proc_ptr->p_reg[REG_DX];
	dx = dx & 0x00ff;
	putk(dx);
	return 0;
};
