%define BASE            0x7c00
%define BUFFER		0x0600
%define BOOTSEG		0x1003
%define BOOTOFF		0x0000
%define LOADSEG		0x1000


section .text	;hay un unico segmento de codigo y datos

;----------------------------------------
;|	inicializo segmentos y punteros		|
;----------------------------------------

                org     BASE		

tarea1:		cli			; Disable maskable Interrupts
		xor	ax,ax		; Zero-out the Accumulator and set
		mov	ss,ax		; Stack Segment register to ZERO.
		mov	sp,BASE		; Stack Pointer top now 0000:7C00
		mov 	ds,ax
		mov	es,ax
		sti
	
		cld		
		mov     si,texto1
		call	printzstr

		mov     di,sectors
		mov	dl,[drive]
		call 	floppy

		xor	ax,ax
		mov	al,cl
		call 	print_int

loadboot:
	        mov    al,[di]         	; al = (di) = sectors per track 	
		mul    dh              	; dh = heads, ax = heads * sectors 	
	        mov    [secpcyl], ax 	; Sectors per cylinder = heads * sectors

	        mov     ax, LOADSEG    	; Segment to load secondary boot code into
	        mov     es, ax
	        xor     bx, bx          ; Load first sector at es:bx = BOOTSEG:0x0000
	        mov     si, addresses  	; Start of the boot code addresses


load: 	
		mov     ax, [si+1]      ; Get next sector number: low 16 bits 	
		mov     dl, [si+3]      ; Bits 16-23 for your 8GB disk
		xor     dh, dh          ; dx:ax = sector within partition

		div     word [secpcyl]  ; ax = cylinder, dx = sector within cylinder
		xchg    ax, dx          ; ax = sector within cylinder, dx = cylinder
		mov    	ch, dl          ; ch = low 8 bits of cylinder 	
		div    	byte [di]       ; al = head, ah = sector (0-origin) 	
		
		xor    	dl, dl          ; About to shift bits 8-9 of cylinder into dl
		shr     dx, 1
		shr     dx, 1          	; dl[6..7] = high cylinder
		or      dl, ah          ; dl[0..5] = sector (0-origin)
		mov    	cl, dl          ; cl[0..5] = sector, cl[6..7] = high cyl
		inc    	cl              ; cl[0..5] = sector (1-origin)
		mov    	dh, al          ; dh = al = head
		mov    	dl, [drive]  	; dl = device to read 	
		mov    	al, [di]        ; Sectors per track - Sector number (0-origin) 	
		sub    	al, ah          ; Sectors left on this track 	
		cmp    	al, [si]        ; Compare with # sectors to read 	
		jbe     read            ; Can't read past the end of a cylinder? 	
		mov    	al, [si]        ; (si) < sectors left on this track 	
		
read:   
		push    ax              ; Save al = sectors to read
		mov    	ah, 2          ; Code for disk read (all registers in use now!) 	
		int     13h            	; Call the BIOS for a read 	

		pop     cx              ; Restore al in cl
		jc      error           ; Jump on disk read error
		mov    	al, cl          ; Restore al = sectors read 	
		add   	bh, al          ; bx += 2 * al * 256 (add bytes read) 	
		add   	bh, al          ; es:bx = where next sector must be read

		add    [si+1], ax       ; Update address by sectors read 	
		sub    [si], al        	; Decrement sector count by sectors read 	

		jnz     load            ; Not all sectors have been read 	
		add     si, 4          ; Next (address, count) pair 	

		cmp    	ah, [si]        ; Done when no sectors to read
		jnz     load            ; Read next chunk of secondary boot code
done: 	
		mov	dl, [drive]
		;jmp 	far [loadaddr]
		jmp	BOOTSEG:BOOTOFF


bucle: 		jmp 	bucle		; bucle infinito


error:
		mov     si,texto_error
		call	printzstr
		jmp	bucle
	

floppy:
		xor    	ah, ah          ; Reset drive 	
		int     13h
		mov    	cl,[di]         ; cl = number of last sector on track

		cmp    	cl, 9          ; No need to do the last 720K/360K test
		je      floppy1

		;Try to read the last sector on track 0
		mov     bx, BUFFER     ; es:bx buffer = 0x0000:0x0600

		mov     ax, 0x0201     ; Read sector, #sectors = 1
		xor    	ch, ch         ; Track 0, last sector
		xor    	dh, dh         ; Drive dl, head 0 	
		int     13h

		jnc     floppy1        
		inc	di
		jmp 	floppy
floppy1:
	        mov	dh,2          	; Load number of heads for multiply
		ret
		

printzstr:	lodsb
		or      al,al		; me fijo si ya esta (AL = 00h)
		jz      printzstr1	; si ya esta, termino
		mov     ah,0Eh		; Video Function 0Eh
		mov     bx,0007		; Page 0, Mode 7
		int     10h		; of INT 10. 'Tele-type Output' AL = character to write.
		jmp     printzstr	; Loop until done
printzstr1:	ret	


print_int:
	        mov     cx,4
print_int_lab1:
		rol     ax,4
		push    ax
		and     ax,000fh
		add     ax,TablaChars
	       	mov     si,ax
	       	lodsb
		push    cx
		mov     ah,0Eh		; Video Function 0Eh
		mov     bx,0007		; Page 0, Mode 7
		int     10h		; of INT 10. 'Tele-type Output' AL = character to write.
		pop    	cx
		pop    	ax
		loop   	print_int_lab1
		ret


drive		db 0
secpcyl 	dw 0

texto1 		db 'Secpcyl: ',0
sectors		db  18, 15, 9
texto_error	db 'Error',0

TablaChars      db  '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'

addresses:
adr_count	db 1
adr_sector	dw 1,0

loadaddr:	dw BOOTOFF,BOOTSEG 

times   	0x01fe-$+$$ db 0	
sign            dw      0xAA55

