extern void print_msg_asm(char *lpsz_msg);
extern void print_int_asm(int);
extern void print_nl_asm();
extern void printk(const char *fmt, ...);
extern void phys_copy( long source, long destination, long bytecount);
extern void jmp_far();

#include "fs.h"
#include "misc.h"

#define MAX_COMMANDCOM_SIZE 8192

#pragma data_seg ("_BSS", "BSS");

int	linker_end;
int	linker_edata;
int	spini;
unsigned int knd_group;
unsigned int code_seg;
unsigned int data_seg;

#define BSS_ARR_SIZE 8
int bss_arr[BSS_ARR_SIZE];
uint8_t p_commandcom[MAX_COMMANDCOM_SIZE];	/* se reserva buffer de carga para command.com*/
uint8_t p_physcopy_buffer[1000];
#pragma data_seg ();

#define CLICK_SHIFT 4

long logic2phys(long segment, long offset)
{
	long phys_addr = (segment << CLICK_SHIFT);
	phys_addr = phys_addr + offset;
	return phys_addr;
};


void main()
{
	int i;

	uint8_t buffer[SECTOR_SIZE];
	uint8_t unit;
	uint32_t count;
	uint32_t cociente, resto;
	unsigned int *p;
	VOLINFO vi;
	uint8_t sector[SECTOR_SIZE];
	FILEINFO fi;
	long phys_dst;
	long phys_src;
	unsigned linear_src_offset, linear_src_segment;
	unsigned linear_dst_offset, linear_dst_segment;


	print_nl_asm();
	print_msg_asm("Hola Mundo desde C!");
	print_nl_asm();
	printk("este string se imprime con printk(char *)\n");
	printk("Esto es el _end en hexa: %X\n",linker_end);
	print_msg_asm("_end: ");
	print_int_asm(linker_end);
	print_nl_asm();
	print_msg_asm("_edata: ");
	print_int_asm(linker_edata);
	print_nl_asm();
	print_msg_asm("sp inicial: ");
	print_int_asm(spini);
	print_nl_asm();

	print_msg_asm("bss_arr: ");

	for (i=0; i<BSS_ARR_SIZE; i++)
	{
		print_int_asm(bss_arr[i]);
		print_msg_asm(" ");
	};
	print_nl_asm();

	printk("Buffer para leer un sector en fs: %X\n", (unsigned int)buffer);

	for (i=0; i<SECTOR_SIZE; i++)
		buffer[i] = 0;

	p = (unsigned int*)(buffer + 510);

	printk("Signature ini: %X\n", *p);

	if (DFS_ReadSector(0, buffer, 0, 1))
	{
		printk("Error leyendo sector de disco");
	}
	else
	{
		printk("Signature end: %X\n", *p);
		/*sector_dump(buffer, 96, FORMATO_HEXA);*/
	};


	if (DFS_GetVolInfo(0, buffer, 0, &vi))
	{
		printk("Error getting volume information\n");
	}
	else
	{
		printk("Volume label '%-11.11s'\n", vi.label);

		printk("%d sector/s per cluster, ", vi.secperclus);
		printk("%d reserved sector/s, ", vi.reservedsecs);
		printk("volume total %d sectors.\n", vi.numsecs);

		printk("%d sectors per FAT, ", vi.secperfat);
		printk("first FAT at sector #%d, ", vi.fat1);
		printk("root dir at #%d.\n", vi.rootdir);

		printk("(For FAT32, the root dir is a CLUSTER number, FAT12/16 it is a SECTOR number)\n");
		printk("%d root dir entries, ", vi.rootentries);
		printk("data area commences at sector #%d.\n",vi.dataarea);
		printk("%d clusters ", vi.numclusters);
		printk("(%d bytes) in data area, filesystem IDd as ", vi.numclusters * vi.secperclus * SECTOR_SIZE);
		if (vi.filesystem == FAT12)
			printk("FAT12.\n");
		else
			printk("[unknown]\n");

	};

	printk("Prueba de lectura de COMMAND.COM\n");	/* Agregado IR */

	if (DFS_OpenFile(&vi, "COMMAND.COM", DFS_READ, sector, &fi))
			printk("Error abriendo archivo file\n");
	else
		{
			unsigned long i;
			DFS_ReadFile(&fi, sector, p_commandcom, &i, fi.filelen);
			printk("Lectura completada: %d bytes leidos, ", i);
			printk("%d bytes esperados.\n", fi.filelen);
			printk("\n");
			printk("Contenido del buffer donde se carga command.com:\n");
			sector_dump(p_commandcom, fi.filelen, FORMATO_HEXA);

		};

	printk("\ncode_seg: %xh\n", code_seg);
	printk("data_seg: %xh\n", data_seg);
	printk("knd_group: %xh\n", knd_group);

	linear_src_offset = (unsigned int)p_commandcom;
	linear_src_segment = data_seg;
	phys_src = logic2phys(linear_src_segment, linear_src_offset);
	printk("\phys_src = %lxh\n", phys_src);

	printk("Prueba de phys_copy\n");
	linear_dst_offset  = (unsigned int)p_physcopy_buffer;
	linear_dst_segment = (unsigned int)data_seg;

	phys_dst = logic2phys(linear_dst_segment, linear_dst_offset);
	printk("phys_dst de 1000h:0100h = %lxh\n", phys_dst);

	phys_copy(phys_src, phys_dst, fi.filelen);

	sector_dump(p_physcopy_buffer, fi.filelen, FORMATO_HEXA);

	for (i=0; i<fi.filelen; i++)
		if (p_physcopy_buffer[i] - p_commandcom[i] != 0)
		{
			printk("Error verificando phys_copy\n.");
			break;
		};

	if (i == fi.filelen)
		printk("\nphys_copy bien\n");

	printk("Cargando buffer a direccion final\n");
	linear_dst_offset = 0x100;
	linear_dst_segment = 0x1000;
	phys_dst = logic2phys(linear_dst_segment, linear_dst_offset);
	printk("phys_dst de 1000h:0100h = %lxh\n", phys_dst);
	phys_copy(phys_src, phys_dst, fi.filelen);

	printk("Y ahora salto...\n");

	jmp_far();

	return;
}
