/* Here is the declaration of the process table.  Three assembly code routines
 * reference fields in it.  They are restart(), save(), and csv().  When
 * changing 'proc', be sure to change the field offsets built into the code.
 * It contains the process' registers, memory map, accounting, and message
 * send/receive information.
 */

#define NR_PROCS 8
#define NR_REGS 11

#define KERNEL_PROC 0
#define COMMAND_PROC 1
#define FIRST_USER_PROC 2

#define SYS_VECTOR 0x21

#define K_STACK_BYTES	256	/* how many bytes for the kernel stack */

#define INIT_PSW      	0x0200	/* initial psw */

#define P_SLOT_FREE		001

typedef unsigned phys_clicks;
typedef long phys_bytes;

struct pc_psw {
  int (*pc)();			/* storage for program counter */
  phys_clicks cs;		/* code segment register */
  unsigned psw;			/* program status word */
};

struct proc {
  int p_reg[NR_REGS];		/* process' registers */
							/* ax, bx, cx, dx, si, di, bp, es, ds, cs, ss*/
  int *p_sp;				/* stack pointer */
  struct pc_psw p_pcpsw;	/* pc and psw as pushed by interrupt */
  int p_flags;				/* P_SLOT_FREE, SENDING, RECEIVING, etc. */
  int p_pid;
  struct proc *p_parent;
};

enum proc_reg
{
	REG_AX,
	REG_BX,
	REG_CX,
	REG_DX,
	REG_SI,
	REG_DI,
	REG_BP,
	REG_ES,
	REG_DS,
	REG_CS,
	REG_SS
};

#pragma data_seg ("_BSS", "BSS");

extern	struct proc proc[NR_PROCS];
extern	struct proc* proc_ptr;
extern 	char k_stack[K_STACK_BYTES];	/* The kernel stack. */

#pragma data_seg ();
