#include "const.h"
#include "syscall.h"
#include "globales.h"
#include "fs.h"
#include "misc.h"
#include "proc.h"
#include "asminc.h"
#include "printk.h"


uint16_t syscall()
{
	int ax, ds, dx, i;
	int nProc;
	char szpath[MAX_PATH];
	ax = proc_ptr->p_reg[REG_AX];
	ds = proc_ptr->p_reg[REG_DS];
	dx = proc_ptr->p_reg[REG_DX];
	ax = ax & 0xff00;
	ax = ax >> 8;
	printk("\nInterrupcion 21h ejecutada con parametro %Xh\n", ax);

	switch (ax)
	{
		case TEST_CODE:
			printk("first_click = %Xh\n", first_click);
			break;
		case TERMINATE_CODE:
			printk("Terminando proceso\n");
			terminate();
			break;
		case EXEC_CODE:
			i = 0;
			for (i=0; i<MAX_PATH; i++)
			{
				szpath[i] = get_byte(ds, dx+i);
				if (szpath[i] == 0)
					break;
			};
			if (i >= MAX_PATH)
				panic("Error EXEC path\n");

			printk("Path to EXEC: %s\n", szpath);
			exec_com(szpath);
			break;
		default:
			break;
	};

	return 1;
};

uint16_t terminate()
{
	uint16_t wError;
	wError = 0;
	if (proc_ptr->p_parent != 0)
	{
		proc_ptr = proc_ptr->p_parent;
	}
	else
	{
		panic("Error terminate\n");
	};
	return wError;
};

uint16_t exec_com(uint8_t *szpath)
{
	uint16_t wError;
	uint8_t buffer[SECTOR_SIZE];
	uint8_t unit;
	uint32_t count;
	unsigned int *p;
	VOLINFO vi;
	uint8_t sector[SECTOR_SIZE];
	FILEINFO fi;
	long phys_dst;
	long phys_src;
	unsigned linear_src_offset, linear_src_segment;
	unsigned linear_dst_offset, linear_dst_segment;
	int i;
	int nProc;
	char c;

	wError = 0;

	if (first_click + COM_MEM_SIZE_CLICKS > MAX_MEM_CLICK)
		return EXEC_COM_ERR_INSUFICIENTMEM;

	for (i=0; i<SECTOR_SIZE; i++)
		buffer[i] = 0;

	p = (unsigned int*)(buffer + 510);

	printk("Signature ini: %X\n", *p);

	if (DFS_ReadSector(0, buffer, 0, 1))
	{
		printk("Error leyendo sector de disco");
	}
	else
	{
		/*printk("Signature end: %X\n", *p);*/
	};

	if (DFS_GetVolInfo(0, buffer, 0, &vi))
	{
		printk("Error getting volume information\n");
	}
	else
	{
/*		printk("Volume label '%-11.11s'\n", vi.label);

		printk("%d sector/s per cluster, ", vi.secperclus);
		printk("%d reserved sector/s, ", vi.reservedsecs);
		printk("volume total %d sectors.\n", vi.numsecs);

		printk("%d sectors per FAT, ", vi.secperfat);
		printk("first FAT at sector #%d, ", vi.fat1);
		printk("root dir at #%d.\n", vi.rootdir);

		printk("(For FAT32, the root dir is a CLUSTER number, FAT12/16 it is a SECTOR number)\n");
		printk("%d root dir entries, ", vi.rootentries);
		printk("data area commences at sector #%d.\n",vi.dataarea);
		printk("%d clusters ", vi.numclusters);
		printk("(%d bytes) in data area, filesystem IDd as ", vi.numclusters * vi.secperclus * SECTOR_SIZE);
		if (vi.filesystem == FAT12)
			printk("FAT12.\n");
		else
			printk("[unknown]\n");*/
	};

	printk("Prueba de lectura de %s\n", szpath);

	if (DFS_OpenFile(&vi, szpath, DFS_READ, sector, &fi))
			printk("Error abriendo archivo file\n");
	else
		{
			unsigned long i;
			DFS_ReadFile(&fi, sector, p_commandcom, &i, fi.filelen);
			printk("Lectura completada: %d bytes leidos, ", i);
			printk("%d bytes esperados.\n", fi.filelen);
			printk("\n");
/*			printk("Contenido del buffer donde se carga %s:\n", szpath);
			sector_dump(p_commandcom, fi.filelen, FORMATO_HEXA);*/
		};


	printk("Cargando buffer a first_click\n");

	linear_src_offset = (unsigned int)p_commandcom;
	linear_src_segment = data_seg;
	phys_src = logic2phys(linear_src_segment, linear_src_offset);
	printk("\phys_src = %lxh\n", phys_src);
	printk("first_click = %Xh\n", first_click);


	linear_dst_offset = 0x100;
	linear_dst_segment = first_click;
	phys_dst = logic2phys(linear_dst_segment, linear_dst_offset);
	printk("phys_dst de first_click:0100h = %lxh\n", phys_dst);
	phys_copy(phys_src, phys_dst, fi.filelen);

	nProc = FIRST_USER_PROC;
	for (i=0; i<NR_PROCS; i++)
	{
		if (proc[i].p_flags & P_SLOT_FREE)
		{
			nProc = i;
			break;
		};
	};

	printk("Primer proceso libre: %X\n", nProc);

	for (i=0; i<REG_ES; i++)
		proc[nProc].p_reg[i] = 0x0000;

	proc[nProc].p_reg[REG_ES] = first_click;
	proc[nProc].p_reg[REG_DS] = first_click;
	proc[nProc].p_reg[REG_CS] = first_click;
	proc[nProc].p_reg[REG_SS] = first_click;

	proc[nProc].p_pcpsw.pc = (int (*)())0x0100;
	proc[nProc].p_pcpsw.psw = INIT_PSW;
	proc[nProc].p_sp = 0;
	proc[nProc].p_flags = 0;

	proc[nProc].p_parent = proc_ptr;

/*	proc_dump(proc_ptr);*/

	first_click += COM_MEM_SIZE_CLICKS;
	proc_ptr = &proc[nProc];

/*	proc_dump(proc_ptr);*/

	printk("first_click a la salida= %Xh\n", first_click);

	return wError;
}

